import React, { ReactNode, useContext, useEffect, useState } from 'react';
import { useRouter } from 'next/router';
import { LoggedUserContext } from '../../contexts/LoggedUserContext';
import { ERole } from '../../api';

export default function Auth({
    minRole,
    children,
}: {
    minRole?: ERole | null;
    children: ReactNode;
}) {
    const { role, tokenExpiration, logout } = useContext(LoggedUserContext);
    const [clientRender, setClientRender] = useState(false);
    const router = useRouter();

    function checkRole(): boolean {
        if (!minRole) {
            // no role required
            return true;
        }

        return role === minRole ?? false;
    }

    useEffect(() => {
        setClientRender(true);
    }, []);

    if (tokenExpiration?.isBefore(new Date())) {
        // token has expired

        (async () => {
            await logout();
            await router.push('/');
        })();
    }

    if (clientRender && checkRole()) {
        return <>{children}</>;
    }

    return null;
}
